@@ -1,6 +1,6 @@
 package QWizard;
 
-our $VERSION = '2.2.2';
+our $VERSION = '2.2.3';
 require Exporter;
 
 our @ISA = qw(Exporter);
@@ -1240,7 +1240,7 @@ sub ask_question {
 					  $self->unparamstr($self->get_value($q->{text})),
 					  $self->{'qcount'});
 
-	$self->do_widget($q, $p);
+	$self->do_widget($q, $p, $def);
 	# return the name
 	$self->{'generator'}->do_question_end($q, $self, $p, $self->{'qcount'});
 	return $q->{'name'};
@@ -2373,6 +2373,38 @@ of the variability in how web-browsers submit file names along with the data,
 this field should generally not be used.  Instead, get access to the data
 through the I<qw_upload_fh>() function instead.
 
+=item filedownload
+
+A button that allows the user to download something generated by the
+application.  The data that will be stored in this file should be
+defined in the 'data' field B<or> the 'datafn' field.  The name
+displayed within the button will be taken from the default question
+parameter.
+
+The I<data> field is processed early during display of the screen, so
+generation of large sets of data that won't always be downloaded or
+will take a lot of memor shouldn't use the I<data> field.  The I<data>
+field is processed like any other value field where raw data or a
+coderef can be passed that will be called to return the data.
+
+The I<datafn> field should contain a I<CODE> reference that will be
+called with two arguments: a IO::File filehandle to print to and the
+file name (if appropriate) it's going to print to.
+
+Example usage:
+
+     { type => 'filedownload',
+       text => 'Download a file:',
+       datafn =>
+       sub { 
+	   my $fh = shift;
+	   print $fh "hello world: val=" . qwparam('someparam') . "\n";
+       }
+     },
+
+Currently only Gtk2 supports this button, but others will in the
+future.
+
 =item image
 
 Image file.  The image name is specified by the I<image> hash keyword.
